@@ -23,6 +23,9 @@ def list_all_regions(session):
 
 def get_role(session, role_name):
     """Returns details about an IAM role"""
+    # We only want the role name if an ARN is passed
+    if "/" in role_name:
+        _, role_name = role_name.split("/", 1)
     try:
         return session.client("iam").get_role(RoleName=role_name)
     except botocore.exceptions.ClientError as e:
@@ -58,7 +61,7 @@ def get_cf_stack_status(session, stack_name):
 
 
 # TODO: Merge this with create_integration_role?
-def create_role(session, role_policy, nr_account_id):
+def create_role(session, role_policy, nr_account_id, tags):
     client = session.client("cloudformation")
     role_policy_name = "" if role_policy is None else role_policy
     stack_name = "NewRelicLambdaIntegrationRole-%d" % nr_account_id
@@ -79,6 +82,9 @@ def create_role(session, role_policy, nr_account_id):
                 {"ParameterKey": "PolicyName", "ParameterValue": role_policy_name},
             ],
             Capabilities=["CAPABILITY_NAMED_IAM"],
+            Tags=[{"Key": key, "Value": value} for key, value in tags]
+            if tags
+            else None,
         )
         click.echo("Waiting for stack creation to complete...", nl=False)
         client.get_waiter("stack_create_complete").wait(StackName=stack_name)
@@ -178,7 +184,14 @@ def import_log_ingestion_function(
 
 
 def create_log_ingestion_function(
-    session, nr_license_key, enable_logs, memory_size, timeout, role_name, mode="CREATE"
+    session,
+    nr_license_key,
+    enable_logs,
+    memory_size,
+    timeout,
+    role_name,
+    mode="CREATE",
+    tags=None,
 ):
     parameters, capabilities = create_log_ingest_parameters(
         nr_license_key, enable_logs, memory_size, timeout, role_name, mode
@@ -198,6 +211,7 @@ def create_log_ingestion_function(
         TemplateURL=template_url,
         Parameters=parameters,
         Capabilities=capabilities,
+        Tags=[{"Key": key, "Value": value} for key, value in tags] if tags else None,
         ChangeSetType=mode,
         ChangeSetName=change_set_name,
     )
@@ -239,7 +253,13 @@ def exec_change_set(cf_client, change_set, mode, stack_name=INGEST_STACK_NAME):
 
 
 def update_log_ingestion_function(
-    session, nr_license_key, enable_logs, memory_size, timeout, role_name=None
+    session,
+    nr_license_key,
+    enable_logs,
+    memory_size,
+    timeout,
+    role_name=None,
+    tags=None,
 ):
     # Detect an old-style nested install and unwrap it
     client = session.client("cloudformation")
@@ -295,6 +315,9 @@ def update_log_ingestion_function(
             TemplateBody=json.dumps(template_body),
             Parameters=params,
             Capabilities=["CAPABILITY_IAM"],
+            Tags=[{"Key": key, "Value": value} for key, value in tags]
+            if tags
+            else None,
         )
         client.get_waiter("stack_update_complete").wait(
             StackName=nested_stack["PhysicalResourceId"]
@@ -347,11 +370,26 @@ def remove_log_ingestion_function(session):
     success("Done")
 
 
-def create_integration_role(session, role_policy, nr_account_id):
+def create_integration_role(session, role_policy, nr_account_id, integration_arn, tags):
     """
     Creates a AWS CloudFormation stack that adds the New Relic AWSLambda Integration
-    IAM role.
+    IAM role. This can be overridden with the `role_arn` parameter, which just checks
+    that the role exists.
     """
+    if integration_arn is not None:
+        role = get_role(session, integration_arn)
+        if role:
+            success(
+                "Found existing AWS IAM role '%s', using it with the New Relic Lambda "
+                "integration" % integration_arn
+            )
+            return role
+        failure(
+            "Could not find AWS IAM role '%s', please verify it exists and run this "
+            "command again" % integration_arn
+        )
+        return
+
     role_name = "NewRelicLambdaIntegrationRole_%s" % nr_account_id
     stack_name = "NewRelicLambdaIntegrationRole-%s" % nr_account_id
     role = get_role(session, role_name)
@@ -360,7 +398,7 @@ def create_integration_role(session, role_policy, nr_account_id):
         return role
     stack_status = get_cf_stack_status(session, stack_name)
     if stack_status is None:
-        create_role(session, role_policy, nr_account_id)
+        create_role(session, role_policy, nr_account_id, tags)
         role = get_role(session, role_name)
         success(
             "Created role [%s] with policy [%s] in AWS account."
@@ -417,6 +455,7 @@ def install_log_ingestion(
     memory_size=128,
     timeout=30,
     role_name=None,
+    tags=None,
 ):
     """
     Installs the New Relic AWS Lambda log ingestion function and role.
@@ -439,6 +478,7 @@ def install_log_ingestion(
                     memory_size,
                     timeout,
                     role_name,
+                    tags,
                 )
             except Exception as e:
                 failure("Failed to create 'newrelic-log-ingestion' function: %s" % e)
@@ -511,9 +551,10 @@ def get_log_ingestion_license_key(session):
     return None
 
 
-def auto_install_license_key(session):
+def auto_install_license_key(session, tags):
     """
-    If the LK secret is missing, create it, picking up the LK value from the ingest lambda's configuration.
+    If the LK secret is missing, create it, picking up the LK value from the ingest
+    lambda's configuration.
     """
     lk_stack_status = get_cf_stack_status(session, LICENSE_KEY_STACK_NAME)
     if lk_stack_status is None:
@@ -522,14 +563,17 @@ def auto_install_license_key(session):
         lk = get_log_ingestion_license_key(session)
         if lk is None:
             failure(
-                "Could not create license key secret; failed to fetch license key value from ingest lambda"
+                "Could not create license key secret; failed to fetch license key "
+                "value from ingest lambda"
             )
             return False
-        return install_license_key(session, nr_license_key=lk)
+        return install_license_key(session, nr_license_key=lk, tags=tags)
     return True
 
 
-def install_license_key(session, nr_license_key, policy_name=None, mode="CREATE"):
+def install_license_key(
+    session, nr_license_key, policy_name=None, mode="CREATE", tags=None
+):
     lk_stack_status = get_cf_stack_status(session, LICENSE_KEY_STACK_NAME)
     if lk_stack_status is None:
         click.echo(
@@ -571,6 +615,9 @@ def install_license_key(session, nr_license_key, policy_name=None, mode="CREATE"
                     TemplateBody=template.read(),
                     Parameters=parameters,
                     Capabilities=["CAPABILITY_NAMED_IAM"],
+                    Tags=[{"Key": key, "Value": value} for key, value in tags]
+                    if tags
+                    else None,
                     ChangeSetType=mode,
                     ChangeSetName=change_set_name,
                 )
@@ -584,12 +631,10 @@ def install_license_key(session, nr_license_key, policy_name=None, mode="CREATE"
     return True
 
 
-def update_license_key(
-    session,
-    nr_license_key,
-    policy_name=None,
-):
-    return install_license_key(session, nr_license_key, policy_name, mode="UPDATE")
+def update_license_key(session, nr_license_key, policy_name=None, tags=None):
+    return install_license_key(
+        session, nr_license_key, policy_name, mode="UPDATE", tags=tags
+    )
 
 
 def remove_license_key(session):
