@@ -5,11 +5,15 @@ import botocore
 import click
 from click.exceptions import Exit
 
-
 NEW_RELIC_ARN_PREFIX_TEMPLATE = "arn:aws:lambda:%s:451483290750"
 RUNTIME_CONFIG = {
+    "dotnetcore3.1": {},
+    "java11": {},
+    "java8.al2": {},
     "nodejs10.x": {"Handler": "newrelic-lambda-wrapper.handler"},
     "nodejs12.x": {"Handler": "newrelic-lambda-wrapper.handler"},
+    "provided": {},
+    "provided.al2": {},
     "python2.7": {"Handler": "newrelic_lambda_wrapper.handler"},
     "python3.6": {"Handler": "newrelic_lambda_wrapper.handler"},
     "python3.7": {"Handler": "newrelic_lambda_wrapper.handler"},
